#version 330

// --- UNIFORMS ---

uniform sampler2D colorTexture;
uniform vec3 LightPosition_worldspace;


// --- INPUT ---

in vec2 fUVs;
in vec3 position_worldspace;
in vec3 normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;

// --- OUTPUT ---

out vec4 fragColor;


void main()
{
	// Light emission properties
	// You probably want to put them as uniforms
	vec3 LightColor = vec3(0.8,0.8,0.75);
	float LightPower = 25.0f;
	
	// Material properties
	vec3 MaterialDiffuseColor = texture( colorTexture, fUVs).rgb;
	vec3 MaterialAmbientColor = vec3(0.3,0.3,0.3) *
	MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.3,0.3,0.3);
	
	// Distance to the light
	float distance = length( LightPosition_worldspace - position_worldspace );
	
	// Normal of the computed fragment, in camera space
	vec3 n = normalize( normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace );
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0a
	float cosTheta = clamp(dot(n,l), 0, 1);
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// clamped to 0
	// Cosine of the angle between the Eye vector and the Reflect vector,
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	fragColor.rgb = /*vec4(*/
		// Ambient : simulates indirect lighting
		MaterialAmbientColor
		// Diffuse : "color" of the object
		+ MaterialDiffuseColor * LightColor * LightPower * cosTheta / (distance*distance)
		// Specular : reflective highlight, like a mirror
		+ MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,5) / (distance*distance)
		/*, 1.0)*/;
	fragColor.a=0.6;
		
}