#version 330

// --- UNIFORMS ---

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;
//uniform vec4 MVP;
uniform vec3 LightPosition_worldspace;

// --- INPUT ---

in vec3 position_modelspace;
in vec3 normal_modelspace;
in vec2 uv;
in vec3 tangent;
in vec3 bitangent;

// --- OUTPUT ---

out vec2 fUVs;
out vec3 position_worldspace;
//out vec3 normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace;
out vec3 EyeDirection_tangentspace;
out vec3 LightDirection_tangentspace;


void main()
{
	// output position of the vertex in projection_space: MVP * position
	gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(position_modelspace, 1);
	
	//Position of the vertex in worldspace
	position_worldspace = (modelMatrix * vec4(position_modelspace,1)).xyz;
	
	// tangent of the vertex in camera space
	vec3 tangent_cameraspace = (viewMatrix * modelMatrix * vec4(tangent, 0)).xyz;

	// bitangent of the vertex in camera space
	vec3 bitangent_cameraspace = (viewMatrix * modelMatrix * vec4(bitangent, 0)).xyz;
	
	// Normal of the the vertex, in camera space
	vec3 normal_cameraspace = (viewMatrix * modelMatrix * vec4(normal_modelspace, 0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
	mat3 TBN = transpose(mat3(tangent_cameraspace,
							  bitangent_cameraspace,
							  normal_cameraspace));
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 position_cameraspace = ( viewMatrix * modelMatrix * vec4(position_modelspace,1)).xyz;
	EyeDirection_cameraspace = vec3(0,0,0) - position_cameraspace;
	EyeDirection_tangentspace = TBN * EyeDirection_cameraspace;
	
	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec3 LightPosition_cameraspace = ( viewMatrix * vec4(LightPosition_worldspace,1)).xyz;
	LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;
	LightDirection_tangentspace = TBN * LightDirection_cameraspace;
	
	//no treatment
	fUVs = uv;
}

